#
# namcap rules - elffiles
# Copyright (C) 2009 Hugo Doria <hugo@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from Namcap.util import is_elf, clean_filename

def scanelf(invalid_elffiles, dirname, names):
	'''Method to scan for ELF files in invalid directories'''

	# Valid directories for ELF files
	valid_dirs = ['/bin/', '/sbin/', '/usr/bin/', '/usr/sbin/', '/lib/', '/usr/lib/']	
	
	for i in names:
		file_path = os.path.join(dirname, i)
		valid_dir_found = False
		
		# Checking for ELF files
		if is_elf(file_path):
			for f in valid_dirs:
				if ('/' + clean_filename(file_path)).startswith(f):
					valid_dir_found = True
					break 

			if not valid_dir_found:
				invalid_elffiles.append(clean_filename(file_path))


class package:
	def short_name(self):
		return "elffiles"
	def long_name(self):
		return "Check about ELF files outside some standard paths."
	def prereq(self):
		return "extract"
	def analyze(self, pkginfo, data):
		ret = [[], [], []]
		invalid_elffiles = []
		
		os.path.walk(data, scanelf, invalid_elffiles)		
		if len(invalid_elffiles) > 0:			
			for i in invalid_elffiles:
				ret[0].append(("elffile-not-in-allowed-dirs %s", i))
					
		return ret
	
	def type(self):
		return "tarball"

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - __init__
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

__tarball__ = """
  anyelf
  capsnamespkg
  depends
  directoryname
  elffiles
  emptydir
  fhsmanpages
  fhsinfopages
  fileownership
  gnomemime
  hicoloricons
  infodirectory
  libtool
  licensepkg
  lotsofdocs
  mimefiles
  perllocal
  permissions
  scrollkeeper
  symlink
  urlpkg

""".split()

__pkgbuild__ = """
  arrays
  badbackups
  capsnames
  carch
  checksums
  invalidstartdir
  license
  pkgname
  rpath
  sfurl
  tags
  url

""".split()

__all__ = __tarball__ + __pkgbuild__

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - fhsmanpages
# Copyright (C) 2008-2009 Aaron Griffin <aaronmgriffin@gmail.com>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import os
import tarfile

class package:
	def short_name(self):
		return "fhs-manpages"
	def long_name(self):
		return "Verifies correct installation of man pages"
	def prereq(self):
		return "tar"
	def analyze(self, pkginfo, tar):
		gooddir = 'usr/share/man'
		bad_dir = 'usr/man'
		ret = [[], [], []]
		for i in tar.getmembers():
			if i.name.startswith(bad_dir):
				ret[0].append(("non-fhs-man-page %s", i.name))
			elif not i.name.startswith(gooddir):
				#Check everything else to see if it has a 'man' path component
				for part in i.name.split(os.sep):
					if part == "man":
						ret[1].append(("potential-non-fhs-man-page %s", i.name))

		return ret
	def type(self):
		return "tarball"
# vim: set ts=4 sw=4 noet:



# 
# namcap rules - mimefiles
# Copyright (C) 2009 Hugo Doria <hugo@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 

class package:
    def short_name(self):
        return "mimefiles"
    def long_name(self):
        return "Check for files in /usr/share/mime"
    def prereq(self):
        return "tar"
    def analyze(self, pkginfo, tar):        
        ret = [[], [], []]
        if 'usr/share/mime' in tar.getnames():            
            if hasattr(pkginfo, "depends"):
                if "shared-mime-info" not in pkginfo.depends:
                    ret[0].append(("dependency-detected-not-included %s", ("shared-mime-info",)))
            if ".INSTALL" not in tar.getnames():
                ret[0].append(("mime-cache-not-updated", ()))
            else:
                f = tar.extractfile(".INSTALL")                    
                if "update-mime-database" not in "\n".join(f.readlines()):
                    ret[0].append(("mime-cache-not-updated", ()))

        return ret
    def type(self):
        return "tarball"



#
# namcap rules - anyelf
# Copyright (C) 2010 Dan McGee <dan@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from Namcap.util import is_elf, clean_filename

def scanelf(found_elffiles, dirname, names):
	'''Look at files and determine if they are ELF files'''

	for i in names:
		file_path = os.path.join(dirname, i)
		# Checking for ELF files
		if is_elf(file_path):
			found_elffiles.append(clean_filename(file_path))

class package:
	def short_name(self):
		return "anyelf"
	def long_name(self):
		return "If package is 'any' architecture, check for ELF files"
	def prereq(self):
		return "extract"
	def analyze(self, pkginfo, data):
		ret = [[], [], []]
		if pkginfo.arch and pkginfo.arch[0] != 'any':
			return ret
		found_elffiles = []

		os.path.walk(data, scanelf, found_elffiles)
		if len(found_elffiles) > 0:
			for i in found_elffiles:
				ret[0].append(("elffile-in-any-package %s", i))

		return ret

	def type(self):
		return "tarball"

# vim: set ts=4 sw=4 noet:



#! /usr/bin/env python
# 
# namcap - A Pacman package analyzer
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# 

# System wide global stuff
import sys, os, os.path, imp, getopt, types, tarfile, re, string, Namcap, pacman
import shutil

sandbox_directory = '/tmp/namcap.' + str(os.getpid())

# Functions
# Return all possible modules (rules)
def get_modules():
	return Namcap.__all__

# Display usage information
def usage():
	print
	print "Usage: " + sys.argv[0] + " [OPTIONS] packages"
	print 
	print "Options are:"
	print "    -i                               : prints information responses from rules"
	print "    -m                               : makes the output parseable (machine-readable)"
	print "    -e rulelist, --exclude=rulelist  : don't apply RULELIST rules to the package"	
	print "    -r rulelist, --rules=rulelist    : only apply RULELIST rules to the package"
	
	
	sys.exit(2)

# Is the package a valid package file?
def verify_package(filename):
	if not os.path.isfile(filename):
		return 0
	if not tarfile.is_tarfile(filename):
		return 0
	try:
		tar = tarfile.open(package, "r")
		if not tar:
			return 0		
		if not len(tar.getmembers()) > 0:
			tar.close()
			return 0
		if not '.PKGINFO' in tar.getnames():
			tar.close()
			return 0
	except IOError:
		if tar:
			tar.close()
		return 0
	return tar

def process_tags(filename=None, machine=False):
	if machine:
		return lambda s: s
	tags = {}
	if not filename:
		filename="/usr/share/namcap/namcap-tags"
	f = open(filename)
	for i in f.readlines():
		if i[0] == "#" or i.strip() == "": continue
		tagdata = i[:-1].split("::")
		tags[tagdata[0].strip()] = tagdata[1].strip()

	return lambda s: tags[s]

def check_rules_exclude(optlist):	
	'''Check if the -r (--rules) and the -r (--exclude) options
	are being used at same time'''
	args_used = 0
	for i in optlist:	
		if '-r' in i or '-e' in i:
			args_used += 1
		if '--rules' in i or '--exclude' in i:
			args_used += 1			
	return args_used

def show_messages(name, key, messages):
	for msg in messages:
		print "%s %s: %s" % (name, key, m(msg[0]) % msg[1])

def process_realpackage(package, modules):
	extracted = 0
	pkgtar = verify_package(package)

	if not pkgtar:
		print "Error: " + package + " is empty or is not a valid package"
		return 1

	pkginfo = pacman.load(package)

	# Loop through each one, load them apply if possible
	for i in modules:
		cur_class = __import__('Namcap.' + i, globals(), locals(), [Namcap])
		pkg = cur_class.package()
		ret = [[], [], []]
		if pkg.type() == "tarball":
			if pkg.prereq() == "extract":
				# If it's not extracted, then extract it and then analyze the package
				if not extracted:
					os.mkdir(sandbox_directory)
					for j in pkgtar.getmembers():
						pkgtar.extract(j, sandbox_directory)
					extracted = 1
				ret = pkg.analyze(pkginfo, sandbox_directory)
			elif pkg.prereq() == "pkg":
				ret = pkg.analyze(pkginfo, None)
			elif pkg.prereq() == "tar":
				ret = pkg.analyze(pkginfo, pkgtar)
			else:
				ret = [['Error running rule (' + i + ')'], [], []]

			# Output the three types of messages
			show_messages(pkginfo.name, 'E', ret[0])
			show_messages(pkginfo.name, 'W', ret[1])
			if info_reporting:
				show_messages(pkginfo.name, 'I', ret[2])

	# Clean up if we extracted anything
	if extracted:
		shutil.rmtree(sandbox_directory)

def process_pkgbuild(package, modules):
	# We might want to do some verifying in here... but really... isn't that what pacman.load is for?
	pkginfo = pacman.load(package)

	if pkginfo == None:
		print "Error: " + package + " is not a valid PKGBUILD"
		return 1

	for i in modules:
		cur_class = __import__('Namcap.' + i, globals(), locals(), [Namcap])
		pkg = cur_class.package()
		ret = [[], [], []]
		if pkg.type() == "pkgbuild":
			ret = pkg.analyze(pkginfo, package)

		# Output the PKGBUILD messages
		name = "PKGBUILD (" + pkginfo.name + ")"
		show_messages(name, 'E', ret[0])
		show_messages(name, 'W', ret[1])
		if info_reporting:
			show_messages(name, 'I', ret[2])


# Main
modules = get_modules()
info_reporting = 0
machine_readable = False
filename = None

# get our options and process them
try:
	optlist, args = getopt.getopt(sys.argv[1:], "ihmr:e:t:",
			["info", "help", "machine-readable", "rules=", "exclude=", "tags="])
except getopt.GetoptError:
	usage()

active_modules = []

# Verifying if we are using the -r and -r options at same time
if check_rules_exclude(optlist) > 1:
	print "You cannot use '-r' (--rules) and '-e' (-exclude) options at same time"
	usage()

for i, k in optlist:
	if i in ('-r', '--rules') and active_modules == []:
		if k == 'list':
			print "-"*20 + " Namcap rule list " + "-"*20
			for j in modules:
				print string.ljust(j, 20) + ": " + __import__('Namcap.' + j, globals(), locals(), [Namcap]).package().long_name()
			sys.exit(2)

		module_list = k.split(',')
		for j in module_list:
			if j in modules:
				active_modules.append(j)
			else:
				print "Error: Rule '" + j + "' does not exist"
				usage()

	# Used to exclude some rules from the check
	if i in ('-e', '--exclude') and active_modules == []:
		module_list = k.split(',')
		active_modules = modules
		for j in module_list:
			if j in modules:
				active_modules.remove(j)
			else:
				print "Error: Rule '" + j + "' does not exist"
				usage()

	if i in ('-i', '--info'):
		info_reporting = 1

	if i in ('-h', '--help'):
		usage()
	if i in ('-m', '--machine-readable'):
		machine_readable = True

	if i in ('-t', '--tags'):
		filename = k

# If there are no args, print usage
if (args == []):
	usage()

m = process_tags(filename=filename, machine=machine_readable)
packages = args

# No rules selected?  Then select them all!
if active_modules == []:
	active_modules = modules

# Go through each package, get the info, and apply the rules
for package in packages:
	if not os.access(package, os.R_OK):
		print "Error: Problem reading " + package
		usage()

	if os.path.isfile(package) and tarfile.is_tarfile(package):
		process_realpackage(package, active_modules)
	elif package[-8:] == 'PKGBUILD':
		process_pkgbuild(package, active_modules)
	else:
		print "Error: Cannot process %s" % package

# vim: set ts=4 sw=4 noet:



#
# namcap rules - utility functions
# Copyright (C) 2009 Dan McGee <dan@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import re

def is_elf(path):
	"""
	Given a file path, ensure it exists and peek at the first few bytes
	to determine if it is an ELF file.
	"""
	if not os.path.isfile(path):
		return False
	fd = open(path)
	bytes = fd.read(4)
	fd.close()
	# magic elf header, present in binaries and libraries
	if bytes == "\x7FELF":
		return True
	else:
		return False

clean_filename = lambda s: re.search(r"/tmp/namcap\.[0-9]*/(.*)", s).group(1)

# vim: set ts=4 sw=4 noet:



#!/usr/bin/env python
from distutils.core import setup

DATAFILES = [('/usr/share/man/man1', ['namcap.1']),
		('/usr/share/namcap', ['namcap-tags']),
		('/usr/share/doc/namcap',['README','AUTHORS','TODO'])]

setup(name="namcap",
	version="2.6",
	description="Pacman package analyzer",
	author="Jason Chu",
	author_email="jason@archlinux.org",
	py_modules=["pacman", "namcap"],
	packages=["Namcap"],
	scripts=["namcap", 'parsepkgbuild'],
	data_files=DATAFILES)

# vim: set ts=4 sw=4 noet:
